import 'dart:math';

import 'package:flutter/material.dart';

import '../entities/cell.dart';
import '../layout/board.dart';
import '../provider/data.dart';
import '../utils/board_utils.dart';
import '../utils/game_utils.dart';

class Game {
  static Container buildGameWidget(Data myProvider) {
    bool gameIsFinished = BoardUtils.checkBoardIsSolved(myProvider);

    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Board.buildGameBoard(myProvider),
          SizedBox(height: 2),
          gameIsFinished
              ? Game.buildEndGameMessage(myProvider)
              : Game.buildSelectCellValueBar(myProvider),
        ],
      ),
    );
  }

  static Container buildSelectCellValueBar(Data myProvider) {
    List cells = myProvider.cells;

    bool isCellSelected =
        (myProvider.currentCellCol != null && myProvider.currentCellRow != null);
    bool isUpdatableCellSelected = isCellSelected
        ? !cells[myProvider.currentCellRow ?? 0][myProvider.currentCellCol ?? 0].isFixed
        : false;
    int maxValue = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    int maxItemsPerLine = 10;
    int linesCount = (maxValue / maxItemsPerLine).floor() + 1;
    int itemsCountPerLine = min(maxItemsPerLine, maxValue + 1);

    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          for (var lineIndex = 0; lineIndex < linesCount; lineIndex++)
            TableRow(
              children: [
                for (var value = lineIndex * itemsCountPerLine;
                    value < (lineIndex + 1) * itemsCountPerLine;
                    value++)
                  Column(
                    children: [
                      Cell(isUpdatableCellSelected ? (value <= maxValue ? value : -1) : -1,
                              false)
                          .widgetUpdateValue(myProvider)
                    ],
                  ),
              ],
            ),
        ],
      ),
    );
  }

  static TextButton buildRestartGameButton(Data myProvider) {
    return TextButton(
      child: Container(
        child: Image(
          image: AssetImage('assets/icons/button_back.png'),
          fit: BoxFit.fill,
        ),
      ),
      onPressed: () => GameUtils.resetGame(myProvider),
    );
  }

  static Container buildEndGameMessage(Data myProvider) {
    Image decorationImage = Image(
      image: AssetImage('assets/icons/game_win.png'),
      fit: BoxFit.fill,
    );

    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              Column(
                children: [decorationImage],
              ),
              Column(
                children: [
                  myProvider.animationInProgress
                      ? decorationImage
                      : buildRestartGameButton(myProvider)
                ],
              ),
              Column(
                children: [decorationImage],
              ),
            ],
          ),
        ],
      ),
    );
  }
}
