import 'package:flutter/material.dart';

import '../provider/data.dart';
import '../utils/game_utils.dart';

class Parameters {
  static double separatorHeight = 2.0;
  static double blockMargin = 3.0;
  static double blockPadding = 2.0;
  static Color buttonBackgroundColor = Colors.white;
  static Color buttonBorderColorActive = Colors.blue;
  static Color buttonBorderColorInactive = Colors.white;
  static double buttonBorderWidth = 10.0;
  static double buttonBorderRadius = 8.0;
  static double buttonPadding = 0.0;
  static double buttonMargin = 0.0;

  static Container buildParametersSelector(Data myProvider) {
    List<Widget> lines = [];

    List parameters = myProvider.availableParameters;
    for (var index = 0; index < parameters.length; index++) {
      lines.add(Parameters.buildParameterSelector(myProvider, parameters[index]));
      lines.add(SizedBox(height: Parameters.separatorHeight));
    }

    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          SizedBox(height: Parameters.separatorHeight),
          Expanded(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: lines,
            ),
          ),
          SizedBox(height: Parameters.separatorHeight),
          Container(
            child: Parameters.buildStartGameButton(myProvider),
          ),
        ],
      ),
    );
  }

  static Container buildStartGameButton(Data myProvider) {
    Column decorationImage = Column(
      children: [
        Image(image: AssetImage('assets/icons/placeholder.png'), fit: BoxFit.fill),
      ],
    );

    return Container(
      margin: EdgeInsets.all(Parameters.blockMargin),
      padding: EdgeInsets.all(Parameters.blockPadding),
      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              decorationImage,
              Column(
                children: [
                  TextButton(
                    child: Container(
                      child: Image(
                        image: AssetImage('assets/icons/button_start.png'),
                        fit: BoxFit.fill,
                      ),
                    ),
                    onPressed: () => GameUtils.startGame(myProvider),
                  ),
                ],
              ),
              decorationImage,
            ],
          ),
        ],
      ),
    );
  }

  static Widget buildParameterSelector(Data myProvider, String parameterCode) {
    List availableValues = myProvider.getParameterAvailableValues(parameterCode);

    if (availableValues.length == 1) {
      return SizedBox(height: 0.0);
    }

    return Table(
      defaultColumnWidth: IntrinsicColumnWidth(),
      children: [
        TableRow(
          children: [
            for (var index = 0; index < availableValues.length; index++)
              Column(
                children: [
                  _buildParameterButton(myProvider, parameterCode, availableValues[index])
                ],
              ),
          ],
        ),
      ],
    );
  }

  static Widget _buildParameterButton(
      Data myProvider, String parameterCode, String parameterValue) {
    String currentValue = myProvider.getParameterValue(parameterCode).toString();

    bool isActive = (parameterValue == currentValue);
    String imageAsset = 'assets/icons/' + parameterCode + '_' + parameterValue + '.png';

    return TextButton(
      child: Container(
        margin: EdgeInsets.all(Parameters.buttonMargin),
        padding: EdgeInsets.all(Parameters.buttonPadding),
        decoration: BoxDecoration(
          color: Parameters.buttonBackgroundColor,
          borderRadius: BorderRadius.circular(Parameters.buttonBorderRadius),
          border: Border.all(
            color: isActive
                ? Parameters.buttonBorderColorActive
                : Parameters.buttonBorderColorInactive,
            width: Parameters.buttonBorderWidth,
          ),
        ),
        child: Image(
          image: AssetImage(imageAsset),
          fit: BoxFit.fill,
        ),
      ),
      onPressed: () => myProvider.setParameterValue(parameterCode, parameterValue),
    );
  }
}
